########################################################################################################################
# Define an atomic test. This class has to be inherited in child classes, where the developer writes test code.
########################################################################################################################

require_relative 'abstract_reporter'
require_relative 'censor'

module Rproof
  class Test
    def initialize(reporter, name, description = nil)
      @id = object_id
      @reporter = reporter
      @name = name
      @description = description
      @censor = Censor.new reporter, name, description
    end

    def setup
    end

    def run
      raise "This method has to be overriden. Please redefine 'run' method with your test code."
    end

    def clean_up
    end

    def execute
      @reporter.report_test_begin(@id, @name, @description)
      begin
        setup
        run
        clean_up
      rescue Exception => e
        @censor.log_exception e
      end
      @reporter.report_test_end(@id, @censor.test_result)
      @censor.test_result
    end
  end
end
