require_relative 'assertion'

module Rproof
  class TestResult
    def initialize(name, description)
      @name = name
      @description = description
      @assertions = []
      @warnings = []
      @exceptions = []
    end
    attr_reader :name, :description, :assertions, :warnings, :exceptions

    def add_assertion(assertion)
      @assertions << assertion
    end

    def add_warning(warning)
      @warnings << warning
    end

    def add_exception(exception)
      @exceptions << exception
    end

    def successes_nb
      successes.count
    end

    def failures_nb
      failures.count
    end

    def successes
      @assertions.select { |assertion| assertion.is_successful }
    end

    def failures
      @assertions.select { |assertion| not assertion.is_successful }
    end

    def status
      if @exceptions.count > 0
        :exception
      elsif failures_nb > 0
        :failed
      else
        :succeed
      end
    end

    # Return worse status between the two
    def self.get_worse_status(status_1, status_2)
      if [status_1, status_2].include? :exception
        :exception
      elsif [status_1, status_2].include? :failed
        :failed
      else
        :succeed
      end
    end
  end
end
