########################################################################################################################
#
########################################################################################################################

require 'builder'

require_relative 'abstract_reporter'
require_relative 'assertion'

module Rproof
  class XMLReporter < AbstractReporter

    def initialize(xml_pathname:)
      @xml_pathname = xml_pathname
    end

    def report_campaign_begin
    end

    def report_suite_begin(id, name, description)
    end

    def report_test_begin(id, name, description)
    end

    def report_test_end(id, test_result)
    end

    def report_suite_end(id, test_results)
    end

    def assertion_to_xml(assertion, xml_assertion)
      xml_assertion.file = assertion.file
      xml_assertion.line = assertion.line
      xml_assertion.method = assertion.method
      xml_assertion.comment = assertion.comment
      xml_assertion.expected = assertion.expected.inspect
      xml_assertion.obtained = assertion.obtained.inspect
      xml_assertion
    end

    def report_campaign_end(test_results, start_time, end_time)
      results = (test_results.is_a? Array) ? test_results.flatten : [test_results]
      xml = Builder::XmlMarkup.new(indent: 2)
      xml.test_campgain do |xml_campgain|
        results.each do |result|
          xml_campgain.test_result do |xml_result|
            xml_result.name result.name
            xml_result.description result.description
            xml_result.assertions do |xml_assertions|
              result.assertions.each do |assertion|
                if assertion.is_successful
                  xml_assertions.success do |xml_assertion|
                    xml_assertion = assertion_to_xml(assertion, xml_assertion)
                  end
                else
                  xml_assertions.failure do |xml_assertion|
                    xml_assertion = assertion_to_xml(assertion, xml_assertion)
                  end
                end
              end
            end
            xml_result.warnings do |xml_warnings|
              result.warnings.each do |warning|
                xml_warnings.warning warning.message
              end
            end
            xml_result.exceptions do |xml_exceptions|
              result.exceptions.each do |exception|
                xml_exceptions.exception do |xml_exception|
                  xml_exception.message = exception.message
                  xml_exception.backtrace = exception.backtrace.join("\n")
                end
              end
            end
          end
        end
      end

      xml_file = File.open(@xml_pathname, "w")
      xml_file << xml.target!
      xml_file.close
    end

  end
end
